@@ -24,6 +24,8 @@
 #include "system/filesys.h"
 #include "system/syslog.h"
 #include "system/locale.h"
+#include "system/network.h"
+#include "system/time.h"
 #include "time_basic.h"
 #include "close_low_fd.h"
 #include "memory.h"
@@ -89,12 +91,17 @@ static struct {
 	bool initialized;
 	enum debug_logtype logtype; /* The type of logging we are doing: eg stdout, file, stderr */
 	char prog_name[255];
+	char hostname[HOST_NAME_MAX+1];
 	bool reopening_logs;
 	bool schedule_reopen_logs;
 
 	struct debug_settings settings;
 	debug_callback_fn callback;
 	void *callback_private;
+	char header_str[300];
+	char header_str_no_nl[300];
+	size_t hs_len;
+	char msg_no_nl[FORMAT_BUFR_SIZE];
 } state = {
 	.settings = {
 		.timestamp_logs = true
@@ -117,46 +124,11 @@ struct debug_class {
 	ino_t ino;
 };
 
-static const char *default_classname_table[] = {
-	[DBGC_ALL] =			"all",
-	[DBGC_TDB] =			"tdb",
-	[DBGC_PRINTDRIVERS] =		"printdrivers",
-	[DBGC_LANMAN] =			"lanman",
-	[DBGC_SMB] =			"smb",
-	[DBGC_RPC_PARSE] =		"rpc_parse",
-	[DBGC_RPC_SRV] =		"rpc_srv",
-	[DBGC_RPC_CLI] =		"rpc_cli",
-	[DBGC_PASSDB] =			"passdb",
-	[DBGC_SAM] =			"sam",
-	[DBGC_AUTH] =			"auth",
-	[DBGC_WINBIND] =		"winbind",
-	[DBGC_VFS] =			"vfs",
-	[DBGC_IDMAP] =			"idmap",
-	[DBGC_QUOTA] =			"quota",
-	[DBGC_ACLS] =			"acls",
-	[DBGC_LOCKING] =		"locking",
-	[DBGC_MSDFS] =			"msdfs",
-	[DBGC_DMAPI] =			"dmapi",
-	[DBGC_REGISTRY] =		"registry",
-	[DBGC_SCAVENGER] =		"scavenger",
-	[DBGC_DNS] =			"dns",
-	[DBGC_LDB] =			"ldb",
-	[DBGC_TEVENT] =			"tevent",
-	[DBGC_AUTH_AUDIT] =		"auth_audit",
-	[DBGC_AUTH_AUDIT_JSON] =	"auth_json_audit",
-	[DBGC_KERBEROS] =       	"kerberos",
-	[DBGC_DRS_REPL] =       	"drs_repl",
-	[DBGC_SMB2] =           	"smb2",
-	[DBGC_SMB2_CREDITS] =   	"smb2_credits",
-	[DBGC_DSDB_AUDIT] =		"dsdb_audit",
-	[DBGC_DSDB_AUDIT_JSON] =	"dsdb_json_audit",
-	[DBGC_DSDB_PWD_AUDIT]  =	"dsdb_password_audit",
-	[DBGC_DSDB_PWD_AUDIT_JSON] =	"dsdb_password_json_audit",
-	[DBGC_DSDB_TXN_AUDIT]  =	"dsdb_transaction_audit",
-	[DBGC_DSDB_TXN_AUDIT_JSON] =	"dsdb_transaction_json_audit",
-	[DBGC_DSDB_GROUP_AUDIT] =	"dsdb_group_audit",
-	[DBGC_DSDB_GROUP_AUDIT_JSON] =	"dsdb_group_json_audit",
-};
+/*
+ * default_classname_table[] is read in from debug-classname-table.c
+ * so that test_logging.c can use it too.
+ */
+#include "lib/util/debug-classes/debug-classname-table.c"
 
 /*
  * This is to allow reading of dbgc_config before the debug
@@ -172,6 +144,72 @@ static struct debug_class *dbgc_config = debug_class_list_initial;
 static int current_msg_level = 0;
 static int current_msg_class = 0;
 
+/*
+ * DBG_DEV(): when and how to user it.
+ *
+ * As a developer, you sometimes want verbose logging between point A and
+ * point B, where the relationship between these points is not easily defined
+ * in terms of the call stack.
+ *
+ * For example, you might be interested in what is going on in functions in
+ * lib/util/util_str.c in an ldap worker process after a particular query. If
+ * you use gdb, something will time out and you won't get the full
+ * conversation. If you add fprintf() or DBG_ERR()s to util_str.c, you'll get
+ * a massive flood, and there's a chance one will accidentally slip into a
+ * release and the whole world will flood. DBG_DEV is a solution.
+ *
+ * On start-up, DBG_DEV() is switched OFF. Nothing is printed.
+ *
+ * 1. Add `DBG_DEV("formatted msg %d, etc\n", i);` where needed.
+ *
+ * 2. At each point you want to start debugging, add `debug_developer_enable()`.
+ *
+ * 3. At each point you want debugging to stop, add `debug_developer_disable()`.
+ *
+ * In DEVELOPER builds, the message will be printed at level 0, as with
+ * DBG_ERR(). In production builds, the macro resolves to nothing.
+ *
+ * The messages are printed with a "<function_name>:DEV:<pid>:" prefix.
+ */
+
+static bool debug_developer_is_enabled = false;
+
+bool debug_developer_enabled(void)
+{
+	return debug_developer_is_enabled;
+}
+
+/*
+ * debug_developer_disable() will turn DBG_DEV() on in the current
+ * process and children.
+ */
+void debug_developer_enable(void)
+{
+	debug_developer_is_enabled = true;
+}
+
+/*
+ * debug_developer_disable() will make DBG_DEV() do nothing in the current
+ * process (and children).
+ */
+void debug_developer_disable(void)
+{
+	debug_developer_is_enabled = false;
+}
+
+/*
+ * Within debug.c, DBG_DEV() always writes to stderr, because some functions
+ * here will attempt infinite recursion with normal DEBUG macros.
+ */
+#ifdef DEVELOPER
+#undef DBG_DEV
+#define DBG_DEV(fmt, ...)						\
+	(void)((debug_developer_enabled())				\
+	       && (fprintf(stderr, "%s:DEV:%d: " fmt "%s",		\
+			   __func__, getpid(), ##__VA_ARGS__, "")) )
+#endif
+
+
 #if defined(WITH_SYSLOG) || defined(HAVE_LIBSYSTEMD_JOURNAL) || defined(HAVE_LIBSYSTEMD)
 static int debug_level_to_priority(int level)
 {
@@ -201,14 +239,65 @@ static int debug_level_to_priority(int level)
 }
 #endif
 
+/* -------------------------------------------------------------------------- **
+ * Produce a version of the given buffer without any trailing newlines.
+ */
+#if defined(HAVE_LIBSYSTEMD_JOURNAL) || defined(HAVE_LIBSYSTEMD) || \
+	defined(HAVE_LTTNG_TRACEF) || defined(HAVE_GPFS)
+static void copy_no_nl(char *out,
+		       size_t out_size,
+		       const char *in,
+		       size_t in_len)
+{
+	size_t len;
+	/*
+	 * Some backends already add an extra newline, so also provide
+	 * a buffer without the newline character.
+	 */
+	len = MIN(in_len, out_size - 1);
+	if ((len > 0) && (in[len - 1] == '\n')) {
+		len--;
+	}
+
+	memcpy(out, in, len);
+	out[len] = '\0';
+}
+
+static void ensure_copy_no_nl(char *out,
+			      size_t out_size,
+			      const char *in,
+			      size_t in_len)
+{
+	/*
+	 * Assume out is a static buffer that is reused as a cache.
+	 * If it isn't empty then this has already been done with the
+	 * same input.
+	 */
+	if (out[0] != '\0') {
+		return;
+	}
+
+	copy_no_nl(out, out_size, in, in_len);
+}
+#endif
+
 /* -------------------------------------------------------------------------- **
  * Debug backends. When logging to DEBUG_FILE, send the log entries to
  * all active backends.
  */
 
-static void debug_file_log(int msg_level,
-			   const char *msg, const char *msg_no_nl)
+static void debug_file_log(int msg_level, const char *msg, size_t msg_len)
 {
+	struct iovec iov[] = {
+		{
+			.iov_base = discard_const(state.header_str),
+			.iov_len = state.hs_len,
+		},
+		{
+			.iov_base = discard_const(msg),
+			.iov_len = msg_len,
+		},
+	};
 	ssize_t ret;
 	int fd;
 
@@ -221,7 +310,7 @@ static void debug_file_log(int msg_level,
 	}
 
 	do {
-		ret = write(fd, msg, strlen(msg));
+		ret = writev(fd, iov, ARRAY_SIZE(iov));
 	} while (ret == -1 && errno == EINTR);
 }
 
@@ -248,8 +337,7 @@ static void debug_syslog_reload(bool enabled, bool previously_enabled,
 	}
 }
 
-static void debug_syslog_log(int msg_level,
-			     const char *msg, const char *msg_no_nl)
+static void debug_syslog_log(int msg_level, const char *msg, size_t msg_len)
 {
 	int priority;
 
@@ -261,16 +349,34 @@ static void debug_syslog_log(int msg_level,
 	 */
 	priority |= SYSLOG_FACILITY;
 
+	if (state.hs_len > 0) {
+		syslog(priority, "%s", state.header_str);
+	}
 	syslog(priority, "%s", msg);
 }
 #endif /* WITH_SYSLOG */
 
 #if defined(HAVE_LIBSYSTEMD_JOURNAL) || defined(HAVE_LIBSYSTEMD)
 #include <systemd/sd-journal.h>
-static void debug_systemd_log(int msg_level,
-			      const char *msg, const char *msg_no_nl)
+static void debug_systemd_log(int msg_level, const char *msg, size_t msg_len)
 {
-	sd_journal_send("MESSAGE=%s", msg_no_nl,
+	if (state.hs_len > 0) {
+		ensure_copy_no_nl(state.header_str_no_nl,
+				  sizeof(state.header_str_no_nl),
+				  state.header_str,
+				  state.hs_len);
+		sd_journal_send("MESSAGE=%s",
+				state.header_str_no_nl,
+				"PRIORITY=%d",
+				debug_level_to_priority(msg_level),
+				"LEVEL=%d",
+				msg_level,
+				NULL);
+	}
+	ensure_copy_no_nl(state.msg_no_nl,
+			  sizeof(state.msg_no_nl),
+			  msg, msg_len);
+	sd_journal_send("MESSAGE=%s", state.msg_no_nl,
 			"PRIORITY=%d", debug_level_to_priority(msg_level),
 			"LEVEL=%d", msg_level,
 			NULL);
@@ -279,10 +385,19 @@ static void debug_systemd_log(int msg_level,
 
 #ifdef HAVE_LTTNG_TRACEF
 #include <lttng/tracef.h>
-static void debug_lttng_log(int msg_level,
-			    const char *msg, const char *msg_no_nl)
+static void debug_lttng_log(int msg_level, const char *msg, size_t msg_len)
 {
-	tracef(msg_no_nl);
+	if (state.hs_len > 0) {
+		ensure_copy_no_nl(state.header_str_no_nl,
+				  sizeof(state.header_str_no_nl),
+				  state.header_str,
+				  state.hs_len);
+		tracef(state.header_str_no_nl);
+	}
+	ensure_copy_no_nl(state.msg_no_nl,
+			  sizeof(state.msg_no_nl),
+			  msg, msg_len);
+	tracef(state.msg_no_nl);
 }
 #endif /* WITH_LTTNG_TRACEF */
 
@@ -311,10 +426,19 @@ static void debug_gpfs_reload(bool enabled, bool previously_enabled,
 	}
 }
 
-static void debug_gpfs_log(int msg_level,
-			   const char *msg, const char *msg_no_nl)
+static void debug_gpfs_log(int msg_level, const char *msg, size_t msg_len)
 {
-	gpfswrap_add_trace(msg_level, msg_no_nl);
+	if (state.hs_len > 0) {
+		ensure_copy_no_nl(state.header_str_no_nl,
+				  sizeof(state.header_str_no_nl),
+				  state.header_str,
+				  state.hs_len);
+		gpfswrap_add_trace(msg_level, state.header_str_no_nl);
+	}
+	ensure_copy_no_nl(state.msg_no_nl,
+			  sizeof(state.msg_no_nl),
+			  msg, msg_len);
+	gpfswrap_add_trace(msg_level, state.msg_no_nl);
 }
 #endif /* HAVE_GPFS */
 
@@ -366,11 +490,8 @@ static void debug_ringbuf_reload(bool enabled, bool previously_enabled,
 	}
 }
 
-static void debug_ringbuf_log(int msg_level,
-			      const char *msg,
-			      const char *msg_no_nl)
+static void _debug_ringbuf_log(int msg_level, const char *msg, size_t msg_len)
 {
-	size_t msglen = strlen(msg);
 	size_t allowed_size;
 
 	if (debug_ringbuf == NULL) {
@@ -380,20 +501,28 @@ static void debug_ringbuf_log(int msg_level,
 	/* Ensure the buffer is always \0 terminated */
 	allowed_size = debug_ringbuf_size - 1;
 
-	if (msglen > allowed_size) {
+	if (msg_len > allowed_size) {
 		return;
 	}
 
-	if ((debug_ringbuf_ofs + msglen) < debug_ringbuf_ofs) {
+	if ((debug_ringbuf_ofs + msg_len) < debug_ringbuf_ofs) {
 		return;
 	}
 
-	if ((debug_ringbuf_ofs + msglen) > allowed_size) {
+	if ((debug_ringbuf_ofs + msg_len) > allowed_size) {
 		debug_ringbuf_ofs = 0;
 	}
 
-	memcpy(debug_ringbuf + debug_ringbuf_ofs, msg, msglen);
-	debug_ringbuf_ofs += msglen;
+	memcpy(debug_ringbuf + debug_ringbuf_ofs, msg, msg_len);
+	debug_ringbuf_ofs += msg_len;
+}
+
+static void debug_ringbuf_log(int msg_level, const char *msg, size_t msg_len)
+{
+	if (state.hs_len > 0) {
+		_debug_ringbuf_log(msg_level, state.header_str, state.hs_len);
+	}
+	_debug_ringbuf_log(msg_level, msg, msg_len);
 }
 
 static struct debug_backend {
@@ -402,7 +531,9 @@ static struct debug_backend {
 	int new_log_level;
 	void (*reload)(bool enabled, bool prev_enabled,
 		       const char *prog_name, char *option);
-	void (*log)(int msg_level, const char *msg, const char *msg_no_nl);
+	void (*log)(int msg_level,
+		    const char *msg,
+		    size_t len);
 	char *option;
 } debug_backends[] = {
 	{
@@ -558,29 +689,25 @@ static void debug_set_backends(const char *param)
 	}
 }
 
-static void debug_backends_log(const char *msg, int msg_level)
+static void debug_backends_log(const char *msg, size_t msg_len, int msg_level)
 {
-	char msg_no_nl[FORMAT_BUFR_SIZE];
 	size_t i;
-	size_t len;
 
 	/*
-	 * Some backends already add an extra newline, so also provide
-	 * a buffer without the newline character.
+	 * Some backends already add an extra newline, so initialize a
+	 * buffer without the newline character.  It will be filled by
+	 * the first backend that needs it.
 	 */
-	len = MIN(strlen(msg), FORMAT_BUFR_SIZE - 1);
-	if ((len > 0) && (msg[len - 1] == '\n')) {
-		len--;
-	}
-
-	memcpy(msg_no_nl, msg, len);
-	msg_no_nl[len] = '\0';
+	state.msg_no_nl[0] = '\0';
 
 	for (i = 0; i < ARRAY_SIZE(debug_backends); i++) {
 		if (msg_level <= debug_backends[i].log_level) {
-			debug_backends[i].log(msg_level, msg, msg_no_nl);
+			debug_backends[i].log(msg_level, msg, msg_len);
 		}
 	}
+
+	/* Only log the header once */
+	state.hs_len = 0;
 }
 
 int debuglevel_get_class(size_t idx)
@@ -668,18 +795,15 @@ utility lists registered debug class names's
 
 char *debug_list_class_names_and_levels(void)
 {
-	char *buf = NULL;
+	char *buf = talloc_strdup(NULL, "");
 	size_t i;
 	/* prepare strings */
 	for (i = 0; i < debug_num_classes; i++) {
-		buf = talloc_asprintf_append(buf,
-					     "%s:%d%s",
-					     classname_table[i],
-					     dbgc_config[i].loglevel,
-					     i == (debug_num_classes - 1) ? "\n" : " ");
-		if (buf == NULL) {
-			return NULL;
-		}
+		talloc_asprintf_addbuf(&buf,
+				       "%s:%d%s",
+				       classname_table[i],
+				       dbgc_config[i].loglevel,
+				       i == (debug_num_classes - 1) ? "\n" : " ");
 	}
 	return buf;
 }
@@ -978,16 +1102,54 @@ void debug_set_settings(struct debug_settings *settings,
 	debug_set_backends(logging_param);
 }
 
-/**
-  control the name of the logfile and whether logging will be to stdout, stderr
-  or a file, and set up syslog
+static void ensure_hostname(void)
+{
+	int ret;
 
-  new_log indicates the destination for the debug log (an enum in
-  order of precedence - once set to DEBUG_FILE, it is not possible to
-  reset to DEBUG_STDOUT for example.  This makes it easy to override
-  for debug to stderr on the command line, as the smb.conf cannot
-  reset it back to file-based logging
-*/
+	if (state.hostname[0] != '\0') {
+		return;
+	}
+
+	ret = gethostname(state.hostname, sizeof(state.hostname));
+	if (ret != 0) {
+		strlcpy(state.hostname, "unknown", sizeof(state.hostname));
+		return;
+	}
+
+	/*
+	 * Ensure NUL termination, since POSIX isn't clear about that.
+	 *
+	 * Don't worry about truncating at the first '.' or similar,
+	 * since this is usually not fully qualified.  Trying to
+	 * truncate opens up the multibyte character gates of hell.
+	 */
+	state.hostname[sizeof(state.hostname) - 1] = '\0';
+}
+
+void debug_set_hostname(const char *name)
+{
+	strlcpy(state.hostname, name, sizeof(state.hostname));
+}
+
+/**
+ * Ensure debug logs are initialised.
+ *
+ * setup_logging() is called to direct logging to the correct outputs, whether
+ * those be stderr, stdout, files, or syslog, and set the program name used in
+ * the logs. It can be called multiple times.
+ *
+ * There is an order of precedence to the log type. Once set to DEBUG_FILE, it
+ * cannot be reset DEFAULT_DEBUG_STDERR, but can be set to DEBUG_STDERR, after
+ * which DEBUG_FILE is unavailable). This makes it possible to override for
+ * debug to stderr on the command line, as the smb.conf cannot reset it back
+ * to file-based logging. See enum debug_logtype.
+ *
+ * @param prog_name the program name. Directory path component will be
+ *                  ignored.
+ *
+ * @param new_logtype the requested destination for the debug log,
+ *                    as an enum debug_logtype.
+ */
 void setup_logging(const char *prog_name, enum debug_logtype new_logtype)
 {
 	debug_init();
@@ -1058,9 +1220,8 @@ void debug_set_callback(void *private_ptr, debug_callback_fn fn)
 	}
 }
 
-static void debug_callback_log(const char *msg, int msg_level)
+static void debug_callback_log(const char *msg, size_t msg_len, int msg_level)
 {
-	size_t msg_len = strlen(msg);
 	char msg_copy[msg_len];
 
 	if ((msg_len > 0) && (msg[msg_len-1] == '\n')) {
@@ -1191,7 +1352,8 @@ bool reopen_logs_internal(void)
 	 * If log file was opened or created successfully, take over stderr to
 	 * catch output into logs.
 	 */
-	if (dbgc_config[DBGC_ALL].fd > 0) {
+	if (!state.settings.debug_no_stderr_redirect &&
+	    dbgc_config[DBGC_ALL].fd > 0) {
 		if (dup2(dbgc_config[DBGC_ALL].fd, 2) == -1) {
 			/* Close stderr too, if dup2 can't point it -
 			   at the logfile.  There really isn't much
@@ -1335,21 +1497,13 @@ void check_log_size( void )
 {
 	off_t maxlog;
 
-	/*
-	 *  We need to be root to check/change log-file, skip this and let the main
-	 *  loop check do a new check as root.
-	 */
-
-#if _SAMBA_BUILD_ == 3
-	if (geteuid() != sec_initial_uid())
-#else
-	if( geteuid() != 0)
-#endif
-	{
-		/* We don't check sec_initial_uid() here as it isn't
-		 * available in common code and we don't generally
-		 * want to rotate and the possibly lose logs in
-		 * make test or the build farm */
+	if (geteuid() != 0) {
+		/*
+		 * We need to be root to change the log file (tests use a fake
+		 * geteuid() from third_party/uid_wrapper). Otherwise we skip
+		 * this and let the main smbd loop or some other process do
+		 * the work.
+		 */
 		return;
 	}
 
@@ -1397,10 +1551,10 @@ void check_log_size( void )
 
 /*************************************************************************
  Write an debug message on the debugfile.
- This is called by dbghdr() and format_debug_text().
+ This is called by format_debug_text().
 ************************************************************************/
 
-static void Debug1(const char *msg)
+static void Debug1(const char *msg, size_t msg_len)
 {
 	int old_errno = errno;
 
@@ -1408,7 +1562,7 @@ static void Debug1(const char *msg)
 
 	switch(state.logtype) {
 	case DEBUG_CALLBACK:
-		debug_callback_log(msg, current_msg_level);
+		debug_callback_log(msg, msg_len, current_msg_level);
 		break;
 	case DEBUG_STDOUT:
 	case DEBUG_STDERR:
@@ -1419,12 +1573,12 @@ static void Debug1(const char *msg)
 			do {
 				ret = write(dbgc_config[DBGC_ALL].fd,
 					    msg,
-					    strlen(msg));
+					    msg_len);
 			} while (ret == -1 && errno == EINTR);
 		}
 		break;
 	case DEBUG_FILE:
-		debug_backends_log(msg, current_msg_level);
+		debug_backends_log(msg, msg_len, current_msg_level);
 		break;
 	};
 
@@ -1440,7 +1594,7 @@ static void Debug1(const char *msg)
 static void bufr_print( void )
 {
 	format_bufr[format_pos] = '\0';
-	(void)Debug1(format_bufr);
+	(void)Debug1(format_bufr, format_pos);
 	format_pos = 0;
 }
 
@@ -1486,8 +1640,9 @@ static void format_debug_text( const char *msg )
 		 * continuation indicator.
 		 */
 		if (format_pos >= FORMAT_BUFR_SIZE - 1) {
+			const char cont[] = " +>\n";
 			bufr_print();
-			(void)Debug1( " +>\n" );
+			(void)Debug1(cont , sizeof(cont) - 1);
 		}
 	}
 
@@ -1519,7 +1674,7 @@ bool dbgsetclass(int level, int cls)
 }
 
 /***************************************************************************
- Print a Debug Header.
+ Put a Debug Header into header_str.
 
  Input:  level    - Debug level of the message (not the system-wide debug
                     level. )
@@ -1544,11 +1699,17 @@ bool dbghdrclass(int level, int cls, const char *location, const char *func)
 	/* Ensure we don't lose any real errno value. */
 	int old_errno = errno;
 	bool verbose = false;
-	char header_str[300];
-	size_t hs_len;
 	struct timeval tv;
 	struct timeval_buf tvbuf;
 
+	/*
+	 * This might be overkill, but if another early return is
+	 * added later then initialising these avoids potential
+	 * problems
+	 */
+	state.hs_len = 0;
+	state.header_str[0] = '\0';
+
 	if( format_pos ) {
 		/* This is a fudge.  If there is stuff sitting in the format_bufr, then
 		 * the *right* thing to do is to call
@@ -1573,17 +1734,62 @@ bool dbghdrclass(int level, int cls, const char *location, const char *func)
 	 * not yet loaded, then default to timestamps on.
 	 */
 	if (!(state.settings.timestamp_logs ||
-	      state.settings.debug_prefix_timestamp)) {
+	      state.settings.debug_prefix_timestamp ||
+	      state.settings.debug_syslog_format)) {
 		return true;
 	}
 
 	GetTimeOfDay(&tv);
+
+	if (state.settings.debug_syslog_format) {
+		if (state.settings.debug_hires_timestamp) {
+			timeval_str_buf(&tv, true, true, &tvbuf);
+		} else {
+			time_t t;
+			struct tm *tm;
+
+			t = (time_t)tv.tv_sec;
+			tm = localtime(&t);
+			if (tm != NULL) {
+				size_t len;
+				len = strftime(tvbuf.buf,
+					       sizeof(tvbuf.buf),
+					       "%b %e %T",
+					       tm);
+				if (len == 0) {
+					/* Trigger default time format below */
+					tm = NULL;
+				}
+			}
+			if (tm == NULL) {
+				snprintf(tvbuf.buf,
+					 sizeof(tvbuf.buf),
+					 "%ld seconds since the Epoch", (long)t);
+			}
+		}
+
+		ensure_hostname();
+		state.hs_len = snprintf(state.header_str,
+					sizeof(state.header_str),
+					"%s %.*s %s[%u]: ",
+					tvbuf.buf,
+					(int)(sizeof(state.hostname) - 1),
+					state.hostname,
+					state.prog_name,
+					(unsigned int) getpid());
+
+		goto full;
+	}
+
 	timeval_str_buf(&tv, false, state.settings.debug_hires_timestamp,
 			&tvbuf);
 
-	hs_len = snprintf(header_str, sizeof(header_str), "[%s, %2d",
-			  tvbuf.buf, level);
-	if (hs_len >= sizeof(header_str)) {
+	state.hs_len = snprintf(state.header_str,
+				sizeof(state.header_str),
+				"[%s, %2d",
+				tvbuf.buf,
+				level);
+	if (state.hs_len >= sizeof(state.header_str) - 1) {
 		goto full;
 	}
 
@@ -1592,54 +1798,81 @@ bool dbghdrclass(int level, int cls, const char *location, const char *func)
 	}
 
 	if (verbose || state.settings.debug_pid) {
-		hs_len += snprintf(
-			header_str + hs_len, sizeof(header_str) - hs_len,
-			", pid=%u", (unsigned int)getpid());
-		if (hs_len >= sizeof(header_str)) {
+		state.hs_len += snprintf(state.header_str + state.hs_len,
+					 sizeof(state.header_str) - state.hs_len,
+					 ", pid=%u",
+					 (unsigned int)getpid());
+		if (state.hs_len >= sizeof(state.header_str) - 1) {
 			goto full;
 		}
 	}
 
 	if (verbose || state.settings.debug_uid) {
-		hs_len += snprintf(
-			header_str + hs_len, sizeof(header_str) - hs_len,
-			", effective(%u, %u), real(%u, %u)",
-			(unsigned int)geteuid(), (unsigned int)getegid(),
-			(unsigned int)getuid(), (unsigned int)getgid());
-		if (hs_len >= sizeof(header_str)) {
+		state.hs_len += snprintf(state.header_str + state.hs_len,
+					 sizeof(state.header_str) - state.hs_len,
+					 ", effective(%u, %u), real(%u, %u)",
+					 (unsigned int)geteuid(),
+					 (unsigned int)getegid(),
+					 (unsigned int)getuid(),
+					 (unsigned int)getgid());
+		if (state.hs_len >= sizeof(state.header_str) - 1) {
 			goto full;
 		}
 	}
 
 	if ((verbose || state.settings.debug_class)
 	    && (cls != DBGC_ALL)) {
-		hs_len += snprintf(
-			header_str + hs_len, sizeof(header_str) - hs_len,
-			", class=%s", classname_table[cls]);
-		if (hs_len >= sizeof(header_str)) {
+		state.hs_len += snprintf(state.header_str + state.hs_len,
+					 sizeof(state.header_str) - state.hs_len,
+					 ", class=%s",
+					 classname_table[cls]);
+		if (state.hs_len >= sizeof(state.header_str) - 1) {
 			goto full;
 		}
 	}
 
-	/*
-	 * No +=, see man man strlcat
-	 */
-	hs_len = strlcat(header_str, "] ", sizeof(header_str));
-	if (hs_len >= sizeof(header_str)) {
-		goto full;
+	if (debug_traceid_get() != 0) {
+		state.hs_len += snprintf(state.header_str + state.hs_len,
+					 sizeof(state.header_str) - state.hs_len,
+					 ", traceid=%" PRIu64,
+					 debug_traceid_get());
+		if (state.hs_len >= sizeof(state.header_str) - 1) {
+			goto full;
+		}
 	}
 
+	state.header_str[state.hs_len] = ']';
+	state.hs_len++;
+	if (state.hs_len < sizeof(state.header_str) - 1) {
+		state.header_str[state.hs_len] = ' ';
+		state.hs_len++;
+	}
+	state.header_str[state.hs_len] = '\0';
+
 	if (!state.settings.debug_prefix_timestamp) {
-		hs_len += snprintf(
-			header_str + hs_len, sizeof(header_str) - hs_len,
-			"%s(%s)\n", location, func);
-		if (hs_len >= sizeof(header_str)) {
+		state.hs_len += snprintf(state.header_str + state.hs_len,
+					 sizeof(state.header_str) - state.hs_len,
+					 "%s(%s)\n",
+					 location,
+					 func);
+		if (state.hs_len >= sizeof(state.header_str)) {
 			goto full;
 		}
 	}
 
 full:
-	(void)Debug1(header_str);
+	/*
+	 * Above code never overflows state.header_str and always
+	 * NUL-terminates correctly.  However, state.hs_len can point
+	 * past the end of the buffer to indicate that truncation
+	 * occurred, so fix it if necessary, since state.hs_len is
+	 * expected to be used after return.
+	 */
+	if (state.hs_len >= sizeof(state.header_str)) {
+		state.hs_len = sizeof(state.header_str) - 1;
+	}
+
+	state.header_str_no_nl[0] = '\0';
 
 	errno = old_errno;
 	return( true );
@@ -1691,3 +1924,17 @@ bool dbgtext(const char *format_str, ... )
 
 	return ret;
 }
+
+static uint64_t debug_traceid = 0;
+
+uint64_t debug_traceid_set(uint64_t id)
+{
+    uint64_t old_id = debug_traceid;
+    debug_traceid = id;
+    return old_id;
+}
+
+uint64_t debug_traceid_get(void)
+{
+    return debug_traceid;
+}
