@@ -2461,6 +2461,7 @@ static char winbind_get_separator(struct pwb_context *ctx)
 {
 	wbcErr wbc_status;
 	static struct wbcInterfaceDetails *details = NULL;
+	char result;
 
 	wbc_status = wbcCtxInterfaceDetails(ctx->wbc_ctx, &details);
 	if (!WBC_ERROR_IS_OK(wbc_status)) {
@@ -2474,7 +2475,9 @@ static char winbind_get_separator(struct pwb_context *ctx)
 		return '\0';
 	}
 
-	return details->winbind_separator;
+	result = details->winbind_separator;
+	wbcFreeMemory(details);
+	return result;
 }
 
 
@@ -2746,7 +2749,7 @@ static int openpam_convert_error_code(struct pwb_context *ctx,
 #define pam_error_code(a, b, c) (c)
 #endif
 
-PAM_EXTERN
+_PUBLIC_ PAM_EXTERN
 int pam_sm_authenticate(pam_handle_t *pamh, int flags,
 			int argc, const char **argv)
 {
@@ -2905,7 +2908,7 @@ out:
 	return retval;
 }
 
-PAM_EXTERN
+_PUBLIC_ PAM_EXTERN
 int pam_sm_setcred(pam_handle_t *pamh, int flags,
 		   int argc, const char **argv)
 {
@@ -2957,7 +2960,7 @@ int pam_sm_setcred(pam_handle_t *pamh, int flags,
  * Account management. We want to verify that the account exists
  * before returning PAM_SUCCESS
  */
-PAM_EXTERN
+_PUBLIC_ PAM_EXTERN
 int pam_sm_acct_mgmt(pam_handle_t *pamh, int flags,
 		   int argc, const char **argv)
 {
@@ -3056,7 +3059,7 @@ int pam_sm_acct_mgmt(pam_handle_t *pamh, int flags,
 	return pam_error_code(ctx, PAM_WINBIND_ACCT_MGMT, ret);
 }
 
-PAM_EXTERN
+_PUBLIC_ PAM_EXTERN
 int pam_sm_open_session(pam_handle_t *pamh, int flags,
 			int argc, const char **argv)
 {
@@ -3083,7 +3086,7 @@ int pam_sm_open_session(pam_handle_t *pamh, int flags,
 	return pam_error_code(ctx, PAM_WINBIND_OPEN_SESSION, ret);
 }
 
-PAM_EXTERN
+_PUBLIC_ PAM_EXTERN
 int pam_sm_close_session(pam_handle_t *pamh, int flags,
 			 int argc, const char **argv)
 {
@@ -3155,7 +3158,7 @@ static bool _pam_require_krb5_auth_after_chauthtok(struct pwb_context *ctx,
 }
 
 
-PAM_EXTERN
+_PUBLIC_ PAM_EXTERN
 int pam_sm_chauthtok(pam_handle_t * pamh, int flags,
 		     int argc, const char **argv)
 {
@@ -3226,7 +3229,15 @@ int pam_sm_chauthtok(pam_handle_t * pamh, int flags,
 	 */
 
 	if (flags & PAM_PRELIM_CHECK) {
-		time_t pwdlastset_prelim = 0;
+		time_t *pwdlastset_prelim = NULL;
+
+		pwdlastset_prelim = talloc_zero(NULL, time_t);
+		if (pwdlastset_prelim == NULL) {
+			_pam_log(ctx, LOG_CRIT,
+				 "password - out of memory");
+			ret = PAM_BUF_ERR;
+			goto out;
+		}
 
 		/* instruct user what is happening */
 
@@ -3258,7 +3269,7 @@ int pam_sm_chauthtok(pam_handle_t * pamh, int flags,
 		ret = winbind_auth_request(ctx, user, pass_old,
 					   NULL, NULL, 0,
 					   &error, NULL,
-					   &pwdlastset_prelim, NULL);
+					   pwdlastset_prelim, NULL);
 
 		if (ret != PAM_ACCT_EXPIRED &&
 		    ret != PAM_AUTHTOK_EXPIRED &&
@@ -3269,7 +3280,8 @@ int pam_sm_chauthtok(pam_handle_t * pamh, int flags,
 		}
 
 		pam_set_data(pamh, PAM_WINBIND_PWD_LAST_SET,
-			     (void *)pwdlastset_prelim, NULL);
+			     pwdlastset_prelim,
+			     _pam_winbind_cleanup_func);
 
 		ret = pam_set_item(pamh, PAM_OLDAUTHTOK,
 				   (const void *) pass_old);
@@ -3280,7 +3292,7 @@ int pam_sm_chauthtok(pam_handle_t * pamh, int flags,
 		}
 	} else if (flags & PAM_UPDATE_AUTHTOK) {
 
-		time_t pwdlastset_update = 0;
+		time_t *pwdlastset_update = NULL;
 
 		/*
 		 * obtain the proposed password
@@ -3343,8 +3355,9 @@ int pam_sm_chauthtok(pam_handle_t * pamh, int flags,
 		 * By reaching here we have approved the passwords and must now
 		 * rebuild the password database file.
 		 */
-		pam_get_data(pamh, PAM_WINBIND_PWD_LAST_SET,
-			     (const void **) &pwdlastset_update);
+		pam_get_data(pamh,
+			     PAM_WINBIND_PWD_LAST_SET,
+			     (const void **)&pwdlastset_update);
 
 		/*
 		 * if cached creds were enabled, make sure to set the
@@ -3356,7 +3369,7 @@ int pam_sm_chauthtok(pam_handle_t * pamh, int flags,
 		}
 
 		ret = winbind_chauthtok_request(ctx, user, pass_old,
-						pass_new, pwdlastset_update);
+						pass_new, *pwdlastset_update);
 		if (ret != PAM_SUCCESS) {
 			pass_old = pass_new = NULL;
 			goto out;
