@@ -24,6 +24,7 @@
 #include "lib/util/tevent_unix.h"
 #include "lib/util/tevent_ntstatus.h"
 
+_PUBLIC_
 int vfs_not_implemented_connect(
 			vfs_handle_struct *handle,
 			const char *service,
@@ -33,11 +34,13 @@ int vfs_not_implemented_connect(
 	return -1;
 }
 
+_PUBLIC_
 void vfs_not_implemented_disconnect(vfs_handle_struct *handle)
 {
 	;
 }
 
+_PUBLIC_
 uint64_t vfs_not_implemented_disk_free(vfs_handle_struct *handle,
 				const struct smb_filename *smb_fname,
 				uint64_t *bsize,
@@ -50,6 +53,7 @@ uint64_t vfs_not_implemented_disk_free(vfs_handle_struct *handle,
 	return 0;
 }
 
+_PUBLIC_
 int vfs_not_implemented_get_quota(vfs_handle_struct *handle,
 				const struct smb_filename *smb_fname,
 				enum SMB_QUOTA_TYPE qtype,
@@ -60,6 +64,7 @@ int vfs_not_implemented_get_quota(vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_set_quota(vfs_handle_struct *handle,
 				  enum SMB_QUOTA_TYPE qtype,
 				  unid_t id, SMB_DISK_QUOTA *dq)
@@ -68,6 +73,7 @@ int vfs_not_implemented_set_quota(vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_get_shadow_copy_data(vfs_handle_struct *handle,
 				files_struct *fsp,
 				struct shadow_copy_data *shadow_copy_data,
@@ -77,6 +83,7 @@ int vfs_not_implemented_get_shadow_copy_data(vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_statvfs(struct vfs_handle_struct *handle,
 				const struct smb_filename *smb_fname,
 				struct vfs_statvfs_struct *statbuf)
@@ -85,18 +92,21 @@ int vfs_not_implemented_statvfs(struct vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
 uint32_t vfs_not_implemented_fs_capabilities(struct vfs_handle_struct *handle,
 				enum timestamp_set_resolution *p_ts_res)
 {
 	return 0;
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_get_dfs_referrals(struct vfs_handle_struct *handle,
 					       struct dfs_GetDFSReferral *r)
 {
 	return NT_STATUS_NOT_IMPLEMENTED;
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_create_dfs_pathat(struct vfs_handle_struct *handle,
 				struct files_struct *dirfsp,
 				const struct smb_filename *smb_fname,
@@ -106,6 +116,7 @@ NTSTATUS vfs_not_implemented_create_dfs_pathat(struct vfs_handle_struct *handle,
 	return NT_STATUS_NOT_IMPLEMENTED;
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_read_dfs_pathat(struct vfs_handle_struct *handle,
 				TALLOC_CTX *mem_ctx,
 				struct files_struct *dirfsp,
@@ -116,6 +127,7 @@ NTSTATUS vfs_not_implemented_read_dfs_pathat(struct vfs_handle_struct *handle,
 	return NT_STATUS_NOT_IMPLEMENTED;
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_snap_check_path(struct vfs_handle_struct *handle,
 				TALLOC_CTX *mem_ctx,
 				const char *service_path,
@@ -124,6 +136,7 @@ NTSTATUS vfs_not_implemented_snap_check_path(struct vfs_handle_struct *handle,
 	return NT_STATUS_NOT_SUPPORTED;
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_snap_create(struct vfs_handle_struct *handle,
 					 TALLOC_CTX *mem_ctx,
 					 const char *base_volume,
@@ -135,6 +148,7 @@ NTSTATUS vfs_not_implemented_snap_create(struct vfs_handle_struct *handle,
 	return NT_STATUS_NOT_SUPPORTED;
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_snap_delete(struct vfs_handle_struct *handle,
 					 TALLOC_CTX *mem_ctx,
 					 char *base_path,
@@ -143,6 +157,7 @@ NTSTATUS vfs_not_implemented_snap_delete(struct vfs_handle_struct *handle,
 	return NT_STATUS_NOT_SUPPORTED;
 }
 
+_PUBLIC_
 DIR *vfs_not_implemented_fdopendir(vfs_handle_struct *handle, files_struct *fsp,
 				   const char *mask, uint32_t attr)
 {
@@ -150,6 +165,7 @@ DIR *vfs_not_implemented_fdopendir(vfs_handle_struct *handle, files_struct *fsp,
 	return NULL;
 }
 
+_PUBLIC_
 struct dirent *vfs_not_implemented_readdir(vfs_handle_struct *handle,
 					   struct files_struct *dirfsp,
 					   DIR *dirp,
@@ -159,22 +175,26 @@ struct dirent *vfs_not_implemented_readdir(vfs_handle_struct *handle,
 	return NULL;
 }
 
+_PUBLIC_
 void vfs_not_implemented_seekdir(vfs_handle_struct *handle, DIR *dirp, long offset)
 {
 	;
 }
 
+_PUBLIC_
 long vfs_not_implemented_telldir(vfs_handle_struct *handle, DIR *dirp)
 {
 	errno = ENOSYS;
 	return (long)-1;
 }
 
+_PUBLIC_
 void vfs_not_implemented_rewind_dir(vfs_handle_struct *handle, DIR *dirp)
 {
 	;
 }
 
+_PUBLIC_
 int vfs_not_implemented_mkdirat(vfs_handle_struct *handle,
 		struct files_struct *dirfsp,
 		const struct smb_filename *smb_fname,
@@ -184,25 +204,28 @@ int vfs_not_implemented_mkdirat(vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_closedir(vfs_handle_struct *handle, DIR *dir)
 {
 	errno = ENOSYS;
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_openat(vfs_handle_struct *handle,
 			       const struct files_struct *dirfsp,
 			       const struct smb_filename *smb_fname,
 			       struct files_struct *fsp,
-			       int flags,
-			       mode_t mode)
+			       const struct vfs_open_how *how)
 {
 	errno = ENOSYS;
 	return -1;
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_create_file(struct vfs_handle_struct *handle,
 				struct smb_request *req,
+				struct files_struct *dirsp,
 				struct smb_filename *smb_fname,
 				uint32_t access_mask,
 				uint32_t share_access,
@@ -222,12 +245,14 @@ NTSTATUS vfs_not_implemented_create_file(struct vfs_handle_struct *handle,
 	return NT_STATUS_NOT_IMPLEMENTED;
 }
 
+_PUBLIC_
 int vfs_not_implemented_close_fn(vfs_handle_struct *handle, files_struct *fsp)
 {
 	errno = ENOSYS;
 	return -1;
 }
 
+_PUBLIC_
 ssize_t vfs_not_implemented_pread(vfs_handle_struct *handle, files_struct *fsp,
 				  void *data, size_t n, off_t offset)
 {
@@ -235,6 +260,7 @@ ssize_t vfs_not_implemented_pread(vfs_handle_struct *handle, files_struct *fsp,
 	return -1;
 }
 
+_PUBLIC_
 struct tevent_req *vfs_not_implemented_pread_send(struct vfs_handle_struct *handle,
 						  TALLOC_CTX *mem_ctx,
 						  struct tevent_context *ev,
@@ -244,6 +270,7 @@ struct tevent_req *vfs_not_implemented_pread_send(struct vfs_handle_struct *hand
 	return NULL;
 }
 
+_PUBLIC_
 ssize_t vfs_not_implemented_pread_recv(struct tevent_req *req,
 				       struct vfs_aio_state *vfs_aio_state)
 {
@@ -251,6 +278,7 @@ ssize_t vfs_not_implemented_pread_recv(struct tevent_req *req,
 	return -1;
 }
 
+_PUBLIC_
 ssize_t vfs_not_implemented_pwrite(vfs_handle_struct *handle, files_struct *fsp,
 				   const void *data, size_t n, off_t offset)
 {
@@ -258,6 +286,7 @@ ssize_t vfs_not_implemented_pwrite(vfs_handle_struct *handle, files_struct *fsp,
 	return -1;
 }
 
+_PUBLIC_
 struct tevent_req *vfs_not_implemented_pwrite_send(struct vfs_handle_struct *handle,
 						   TALLOC_CTX *mem_ctx,
 						   struct tevent_context *ev,
@@ -268,6 +297,7 @@ struct tevent_req *vfs_not_implemented_pwrite_send(struct vfs_handle_struct *han
 	return NULL;
 }
 
+_PUBLIC_
 ssize_t vfs_not_implemented_pwrite_recv(struct tevent_req *req,
 				struct vfs_aio_state *vfs_aio_state)
 {
@@ -275,6 +305,7 @@ ssize_t vfs_not_implemented_pwrite_recv(struct tevent_req *req,
 	return -1;
 }
 
+_PUBLIC_
 off_t vfs_not_implemented_lseek(vfs_handle_struct *handle, files_struct *fsp,
 			off_t offset, int whence)
 {
@@ -282,6 +313,7 @@ off_t vfs_not_implemented_lseek(vfs_handle_struct *handle, files_struct *fsp,
 	return (off_t) - 1;
 }
 
+_PUBLIC_
 ssize_t vfs_not_implemented_sendfile(vfs_handle_struct *handle, int tofd,
 				     files_struct *fromfsp, const DATA_BLOB *hdr,
 				     off_t offset, size_t n)
@@ -290,6 +322,7 @@ ssize_t vfs_not_implemented_sendfile(vfs_handle_struct *handle, int tofd,
 	return -1;
 }
 
+_PUBLIC_
 ssize_t vfs_not_implemented_recvfile(vfs_handle_struct *handle, int fromfd,
 				     files_struct *tofsp, off_t offset, size_t n)
 {
@@ -297,6 +330,7 @@ ssize_t vfs_not_implemented_recvfile(vfs_handle_struct *handle, int fromfd,
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_renameat(vfs_handle_struct *handle,
 			       files_struct *srcfsp,
 			       const struct smb_filename *smb_fname_src,
@@ -307,6 +341,7 @@ int vfs_not_implemented_renameat(vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
 struct tevent_req *vfs_not_implemented_fsync_send(struct vfs_handle_struct *handle,
 						  TALLOC_CTX *mem_ctx,
 						  struct tevent_context *ev,
@@ -315,6 +350,7 @@ struct tevent_req *vfs_not_implemented_fsync_send(struct vfs_handle_struct *hand
 	return NULL;
 }
 
+_PUBLIC_
 int vfs_not_implemented_fsync_recv(struct tevent_req *req,
 				   struct vfs_aio_state *vfs_aio_state)
 {
@@ -322,12 +358,14 @@ int vfs_not_implemented_fsync_recv(struct tevent_req *req,
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_stat(vfs_handle_struct *handle, struct smb_filename *smb_fname)
 {
 	errno = ENOSYS;
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_fstat(vfs_handle_struct *handle, files_struct *fsp,
 			SMB_STRUCT_STAT *sbuf)
 {
@@ -335,6 +373,7 @@ int vfs_not_implemented_fstat(vfs_handle_struct *handle, files_struct *fsp,
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_lstat(vfs_handle_struct *handle,
 			      struct smb_filename *smb_fname)
 {
@@ -342,6 +381,19 @@ int vfs_not_implemented_lstat(vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
+int vfs_not_implemented_fstatat(
+	struct vfs_handle_struct *handle,
+	const struct files_struct *dirfsp,
+	const struct smb_filename *smb_fname,
+	SMB_STRUCT_STAT *sbuf,
+	int flags)
+{
+	errno = ENOSYS;
+	return -1;
+}
+
+_PUBLIC_
 uint64_t vfs_not_implemented_get_alloc_size(struct vfs_handle_struct *handle,
 					    struct files_struct *fsp,
 					    const SMB_STRUCT_STAT *sbuf)
@@ -350,6 +402,7 @@ uint64_t vfs_not_implemented_get_alloc_size(struct vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_unlinkat(vfs_handle_struct *handle,
 			struct files_struct *dirfsp,
 			const struct smb_filename *smb_fname,
@@ -359,6 +412,7 @@ int vfs_not_implemented_unlinkat(vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_fchmod(vfs_handle_struct *handle, files_struct *fsp,
 			       mode_t mode)
 {
@@ -366,6 +420,7 @@ int vfs_not_implemented_fchmod(vfs_handle_struct *handle, files_struct *fsp,
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_fchown(vfs_handle_struct *handle, files_struct *fsp,
 			       uid_t uid, gid_t gid)
 {
@@ -373,6 +428,7 @@ int vfs_not_implemented_fchown(vfs_handle_struct *handle, files_struct *fsp,
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_lchown(vfs_handle_struct *handle,
 			       const struct smb_filename *smb_fname,
 			       uid_t uid,
@@ -382,6 +438,7 @@ int vfs_not_implemented_lchown(vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_chdir(vfs_handle_struct *handle,
 			      const struct smb_filename *smb_fname)
 {
@@ -389,6 +446,7 @@ int vfs_not_implemented_chdir(vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
 struct smb_filename *vfs_not_implemented_getwd(vfs_handle_struct *handle,
 					       TALLOC_CTX *ctx)
 {
@@ -396,6 +454,7 @@ struct smb_filename *vfs_not_implemented_getwd(vfs_handle_struct *handle,
 	return NULL;
 }
 
+_PUBLIC_
 int vfs_not_implemented_fntimes(vfs_handle_struct *handle,
 				files_struct *fsp,
 				struct smb_file_time *ft)
@@ -404,6 +463,7 @@ int vfs_not_implemented_fntimes(vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_ftruncate(vfs_handle_struct *handle, files_struct *fsp,
 				  off_t offset)
 {
@@ -411,6 +471,7 @@ int vfs_not_implemented_ftruncate(vfs_handle_struct *handle, files_struct *fsp,
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_fallocate(vfs_handle_struct *handle, files_struct *fsp,
 				  uint32_t mode, off_t offset, off_t len)
 {
@@ -418,6 +479,7 @@ int vfs_not_implemented_fallocate(vfs_handle_struct *handle, files_struct *fsp,
 	return -1;
 }
 
+_PUBLIC_
 bool vfs_not_implemented_lock(vfs_handle_struct *handle, files_struct *fsp, int op,
 			      off_t offset, off_t count, int type)
 {
@@ -425,14 +487,17 @@ bool vfs_not_implemented_lock(vfs_handle_struct *handle, files_struct *fsp, int
 	return false;
 }
 
-int vfs_not_implemented_kernel_flock(struct vfs_handle_struct *handle,
-				     struct files_struct *fsp,
-				     uint32_t share_access, uint32_t access_mask)
+_PUBLIC_
+int vfs_not_implemented_filesystem_sharemode(struct vfs_handle_struct *handle,
+					     struct files_struct *fsp,
+					     uint32_t share_access,
+					     uint32_t access_mask)
 {
 	errno = ENOSYS;
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_fcntl(struct vfs_handle_struct *handle,
 			      struct files_struct *fsp, int cmd,
 			      va_list cmd_arg)
@@ -441,6 +506,7 @@ int vfs_not_implemented_fcntl(struct vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_linux_setlease(struct vfs_handle_struct *handle,
 				       struct files_struct *fsp, int leasetype)
 {
@@ -448,6 +514,7 @@ int vfs_not_implemented_linux_setlease(struct vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
 bool vfs_not_implemented_getlock(vfs_handle_struct *handle, files_struct *fsp,
 				 off_t *poffset, off_t *pcount, int *ptype,
 				 pid_t *ppid)
@@ -456,6 +523,7 @@ bool vfs_not_implemented_getlock(vfs_handle_struct *handle, files_struct *fsp,
 	return false;
 }
 
+_PUBLIC_
 int vfs_not_implemented_symlinkat(vfs_handle_struct *handle,
 				const struct smb_filename *link_contents,
 				struct files_struct *dirfsp,
@@ -465,6 +533,7 @@ int vfs_not_implemented_symlinkat(vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_vfs_readlinkat(vfs_handle_struct *handle,
 			const struct files_struct *dirfsp,
 			const struct smb_filename *smb_fname,
@@ -475,6 +544,7 @@ int vfs_not_implemented_vfs_readlinkat(vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_linkat(vfs_handle_struct *handle,
 			files_struct *srcfsp,
 			const struct smb_filename *old_smb_fname,
@@ -486,6 +556,7 @@ int vfs_not_implemented_linkat(vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_mknodat(vfs_handle_struct *handle,
 			files_struct *dirfsp,
 			const struct smb_filename *smb_fname,
@@ -496,6 +567,7 @@ int vfs_not_implemented_mknodat(vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
 struct smb_filename *vfs_not_implemented_realpath(vfs_handle_struct *handle,
 						  TALLOC_CTX *ctx,
 						  const struct smb_filename *smb_fname)
@@ -504,6 +576,7 @@ struct smb_filename *vfs_not_implemented_realpath(vfs_handle_struct *handle,
 	return NULL;
 }
 
+_PUBLIC_
 int vfs_not_implemented_fchflags(vfs_handle_struct *handle,
 				struct files_struct *fsp,
 				uint flags)
@@ -512,6 +585,7 @@ int vfs_not_implemented_fchflags(vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
 struct file_id vfs_not_implemented_file_id_create(vfs_handle_struct *handle,
 						  const SMB_STRUCT_STAT *sbuf)
 {
@@ -521,6 +595,7 @@ struct file_id vfs_not_implemented_file_id_create(vfs_handle_struct *handle,
 	return id;
 }
 
+_PUBLIC_
 uint64_t vfs_not_implemented_fs_file_id(vfs_handle_struct *handle,
 					const SMB_STRUCT_STAT *sbuf)
 {
@@ -532,6 +607,7 @@ struct vfs_not_implemented_offload_read_state {
 	bool dummy;
 };
 
+_PUBLIC_
 struct tevent_req *vfs_not_implemented_offload_read_send(
 			TALLOC_CTX *mem_ctx,
 			struct tevent_context *ev,
@@ -555,9 +631,12 @@ struct tevent_req *vfs_not_implemented_offload_read_send(
 	return tevent_req_post(req, ev);
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_offload_read_recv(struct tevent_req *req,
 				       struct vfs_handle_struct *handle,
 				       TALLOC_CTX *mem_ctx,
+				       uint32_t *flags,
+				       uint64_t *xferlen,
 				       DATA_BLOB *_token_blob)
 {
 	NTSTATUS status;
@@ -575,6 +654,7 @@ struct vfs_not_implemented_offload_write_state {
 	uint64_t unused;
 };
 
+_PUBLIC_
 struct tevent_req *vfs_not_implemented_offload_write_send(
 			struct vfs_handle_struct *handle,
 			TALLOC_CTX *mem_ctx,
@@ -599,6 +679,7 @@ struct tevent_req *vfs_not_implemented_offload_write_send(
 	return tevent_req_post(req, ev);
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_offload_write_recv(struct vfs_handle_struct *handle,
 						struct tevent_req *req,
 						off_t *copied)
@@ -614,6 +695,7 @@ NTSTATUS vfs_not_implemented_offload_write_recv(struct vfs_handle_struct *handle
 	return NT_STATUS_OK;
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_fget_compression(struct vfs_handle_struct *handle,
 					     TALLOC_CTX *mem_ctx,
 					     struct files_struct *fsp,
@@ -622,6 +704,7 @@ NTSTATUS vfs_not_implemented_fget_compression(struct vfs_handle_struct *handle,
 	return NT_STATUS_INVALID_DEVICE_REQUEST;
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_set_compression(struct vfs_handle_struct *handle,
 					     TALLOC_CTX *mem_ctx,
 					     struct files_struct *fsp,
@@ -630,6 +713,7 @@ NTSTATUS vfs_not_implemented_set_compression(struct vfs_handle_struct *handle,
 	return NT_STATUS_INVALID_DEVICE_REQUEST;
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_fstreaminfo(struct vfs_handle_struct *handle,
 					struct files_struct *fsp,
 					TALLOC_CTX *mem_ctx,
@@ -639,23 +723,28 @@ NTSTATUS vfs_not_implemented_fstreaminfo(struct vfs_handle_struct *handle,
 	return NT_STATUS_NOT_IMPLEMENTED;
 }
 
-int vfs_not_implemented_get_real_filename(struct vfs_handle_struct *handle,
-					  const struct smb_filename *path,
-					  const char *name,
-					  TALLOC_CTX *mem_ctx,
-					  char **found_name)
+_PUBLIC_
+NTSTATUS vfs_not_implemented_get_real_filename_at(
+	struct vfs_handle_struct *handle,
+	struct files_struct *dirfsp,
+	const char *name,
+	TALLOC_CTX *mem_ctx,
+	char **found_name)
 {
-	errno = ENOSYS;
-	return -1;
+	return NT_STATUS_NOT_IMPLEMENTED;
 }
 
-const char *vfs_not_implemented_connectpath(struct vfs_handle_struct *handle,
-					    const struct smb_filename *smb_fname)
+_PUBLIC_
+const char *vfs_not_implemented_connectpath(
+	struct vfs_handle_struct *handle,
+	const struct files_struct *dirfsp,
+	const struct smb_filename *smb_fname)
 {
 	errno = ENOSYS;
 	return NULL;
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_brl_lock_windows(struct vfs_handle_struct *handle,
 					      struct byte_range_lock *br_lck,
 					      struct lock_struct *plock)
@@ -663,6 +752,7 @@ NTSTATUS vfs_not_implemented_brl_lock_windows(struct vfs_handle_struct *handle,
 	return NT_STATUS_NOT_IMPLEMENTED;
 }
 
+_PUBLIC_
 bool vfs_not_implemented_brl_unlock_windows(struct vfs_handle_struct *handle,
 					    struct byte_range_lock *br_lck,
 					    const struct lock_struct *plock)
@@ -671,6 +761,7 @@ bool vfs_not_implemented_brl_unlock_windows(struct vfs_handle_struct *handle,
 	return false;
 }
 
+_PUBLIC_
 bool vfs_not_implemented_strict_lock_check(struct vfs_handle_struct *handle,
 					   struct files_struct *fsp,
 					   struct lock_struct *plock)
@@ -679,6 +770,7 @@ bool vfs_not_implemented_strict_lock_check(struct vfs_handle_struct *handle,
 	return false;
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_translate_name(struct vfs_handle_struct *handle,
 					    const char *mapped_name,
 					    enum vfs_translate_direction direction,
@@ -687,6 +779,7 @@ NTSTATUS vfs_not_implemented_translate_name(struct vfs_handle_struct *handle,
 	return NT_STATUS_NOT_IMPLEMENTED;
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_parent_pathname(struct vfs_handle_struct *handle,
 						    TALLOC_CTX *mem_ctx,
 						    const struct smb_filename *smb_fname_in,
@@ -696,6 +789,7 @@ NTSTATUS vfs_not_implemented_parent_pathname(struct vfs_handle_struct *handle,
 	return NT_STATUS_NOT_IMPLEMENTED;
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_fsctl(struct vfs_handle_struct *handle,
 				   struct files_struct *fsp,
 				   TALLOC_CTX *ctx,
@@ -709,6 +803,7 @@ NTSTATUS vfs_not_implemented_fsctl(struct vfs_handle_struct *handle,
 	return NT_STATUS_NOT_IMPLEMENTED;
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_freaddir_attr(struct vfs_handle_struct *handle,
 					struct files_struct *fsp,
 					TALLOC_CTX *mem_ctx,
@@ -722,6 +817,7 @@ struct vfs_not_implemented_get_dos_attributes_state {
 	uint32_t dosmode;
 };
 
+_PUBLIC_
 struct tevent_req *vfs_not_implemented_get_dos_attributes_send(
 			TALLOC_CTX *mem_ctx,
 			struct tevent_context *ev,
@@ -742,6 +838,7 @@ struct tevent_req *vfs_not_implemented_get_dos_attributes_send(
 	return tevent_req_post(req, ev);
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_get_dos_attributes_recv(
 			struct tevent_req *req,
 			struct vfs_aio_state *aio_state,
@@ -763,6 +860,7 @@ NTSTATUS vfs_not_implemented_get_dos_attributes_recv(
 	return NT_STATUS_OK;
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_fget_dos_attributes(struct vfs_handle_struct *handle,
 						 struct files_struct *fsp,
 						 uint32_t *dosmode)
@@ -770,6 +868,7 @@ NTSTATUS vfs_not_implemented_fget_dos_attributes(struct vfs_handle_struct *handl
 	return NT_STATUS_NOT_IMPLEMENTED;
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_fset_dos_attributes(struct vfs_handle_struct *handle,
 						 struct files_struct *fsp,
 						 uint32_t dosmode)
@@ -777,6 +876,7 @@ NTSTATUS vfs_not_implemented_fset_dos_attributes(struct vfs_handle_struct *handl
 	return NT_STATUS_NOT_IMPLEMENTED;
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_fget_nt_acl(vfs_handle_struct *handle, files_struct *fsp,
 					 uint32_t security_info,
 					 TALLOC_CTX *mem_ctx,
@@ -785,6 +885,7 @@ NTSTATUS vfs_not_implemented_fget_nt_acl(vfs_handle_struct *handle, files_struct
 	return NT_STATUS_NOT_IMPLEMENTED;
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_fset_nt_acl(vfs_handle_struct *handle, files_struct *fsp,
 					 uint32_t security_info_sent,
 					 const struct security_descriptor *psd)
@@ -792,6 +893,7 @@ NTSTATUS vfs_not_implemented_fset_nt_acl(vfs_handle_struct *handle, files_struct
 	return NT_STATUS_NOT_IMPLEMENTED;
 }
 
+_PUBLIC_
 SMB_ACL_T vfs_not_implemented_sys_acl_get_fd(vfs_handle_struct *handle,
 					     files_struct *fsp,
 					     SMB_ACL_TYPE_T type,
@@ -801,6 +903,7 @@ SMB_ACL_T vfs_not_implemented_sys_acl_get_fd(vfs_handle_struct *handle,
 	return (SMB_ACL_T) NULL;
 }
 
+_PUBLIC_
 int vfs_not_implemented_sys_acl_blob_get_fd(vfs_handle_struct *handle,
 				files_struct *fsp, TALLOC_CTX *mem_ctx,
 				char **blob_description, DATA_BLOB *blob)
@@ -809,6 +912,7 @@ int vfs_not_implemented_sys_acl_blob_get_fd(vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_sys_acl_set_fd(vfs_handle_struct *handle,
 				       struct files_struct *fsp,
 				       SMB_ACL_TYPE_T type,
@@ -818,6 +922,7 @@ int vfs_not_implemented_sys_acl_set_fd(vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_sys_acl_delete_def_fd(vfs_handle_struct *handle,
 					struct files_struct *fsp)
 {
@@ -831,6 +936,7 @@ struct vfs_not_implemented_getxattrat_state {
 	uint8_t *xattr_value;
 };
 
+_PUBLIC_
 struct tevent_req *vfs_not_implemented_getxattrat_send(
 			TALLOC_CTX *mem_ctx,
 			struct tevent_context *ev,
@@ -853,6 +959,7 @@ struct tevent_req *vfs_not_implemented_getxattrat_send(
 	return tevent_req_post(req, ev);
 }
 
+_PUBLIC_
 ssize_t vfs_not_implemented_getxattrat_recv(struct tevent_req *req,
 				    struct vfs_aio_state *aio_state,
 				    TALLOC_CTX *mem_ctx,
@@ -877,6 +984,7 @@ ssize_t vfs_not_implemented_getxattrat_recv(struct tevent_req *req,
 	return xattr_size;
 }
 
+_PUBLIC_
 ssize_t vfs_not_implemented_fgetxattr(vfs_handle_struct *handle,
 			      struct files_struct *fsp, const char *name,
 			      void *value, size_t size)
@@ -885,6 +993,7 @@ ssize_t vfs_not_implemented_fgetxattr(vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
 ssize_t vfs_not_implemented_flistxattr(vfs_handle_struct *handle,
 				       struct files_struct *fsp, char *list,
 				       size_t size)
@@ -893,6 +1002,7 @@ ssize_t vfs_not_implemented_flistxattr(vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_fremovexattr(vfs_handle_struct *handle,
 				     struct files_struct *fsp, const char *name)
 {
@@ -900,6 +1010,7 @@ int vfs_not_implemented_fremovexattr(vfs_handle_struct *handle,
 	return -1;
 }
 
+_PUBLIC_
 int vfs_not_implemented_fsetxattr(vfs_handle_struct *handle, struct files_struct *fsp,
 				  const char *name, const void *value, size_t size,
 				  int flags)
@@ -908,6 +1019,7 @@ int vfs_not_implemented_fsetxattr(vfs_handle_struct *handle, struct files_struct
 	return -1;
 }
 
+_PUBLIC_
 bool vfs_not_implemented_aio_force(struct vfs_handle_struct *handle,
 				   struct files_struct *fsp)
 {
@@ -915,6 +1027,7 @@ bool vfs_not_implemented_aio_force(struct vfs_handle_struct *handle,
 	return false;
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_audit_file(struct vfs_handle_struct *handle,
 					struct smb_filename *file,
 					struct security_acl *sacl,
@@ -924,6 +1037,7 @@ NTSTATUS vfs_not_implemented_audit_file(struct vfs_handle_struct *handle,
 	return NT_STATUS_NOT_IMPLEMENTED;
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_durable_cookie(struct vfs_handle_struct *handle,
 					    struct files_struct *fsp,
 					    TALLOC_CTX *mem_ctx,
@@ -932,6 +1046,7 @@ NTSTATUS vfs_not_implemented_durable_cookie(struct vfs_handle_struct *handle,
 	return NT_STATUS_NOT_IMPLEMENTED;
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_durable_disconnect(struct vfs_handle_struct *handle,
 						struct files_struct *fsp,
 						const DATA_BLOB old_cookie,
@@ -941,6 +1056,7 @@ NTSTATUS vfs_not_implemented_durable_disconnect(struct vfs_handle_struct *handle
 	return NT_STATUS_NOT_IMPLEMENTED;
 }
 
+_PUBLIC_
 NTSTATUS vfs_not_implemented_durable_reconnect(struct vfs_handle_struct *handle,
 					       struct smb_request *smb1req,
 					       struct smbXsrv_open *op,
@@ -1002,6 +1118,7 @@ static struct vfs_fn_pointers vfs_not_implemented_fns = {
 	.stat_fn = vfs_not_implemented_stat,
 	.fstat_fn = vfs_not_implemented_fstat,
 	.lstat_fn = vfs_not_implemented_lstat,
+	.fstatat_fn = vfs_not_implemented_fstatat,
 	.get_alloc_size_fn = vfs_not_implemented_get_alloc_size,
 	.unlinkat_fn = vfs_not_implemented_unlinkat,
 	.fchmod_fn = vfs_not_implemented_fchmod,
@@ -1013,7 +1130,7 @@ static struct vfs_fn_pointers vfs_not_implemented_fns = {
 	.ftruncate_fn = vfs_not_implemented_ftruncate,
 	.fallocate_fn = vfs_not_implemented_fallocate,
 	.lock_fn = vfs_not_implemented_lock,
-	.kernel_flock_fn = vfs_not_implemented_kernel_flock,
+	.filesystem_sharemode_fn = vfs_not_implemented_filesystem_sharemode,
 	.fcntl_fn = vfs_not_implemented_fcntl,
 	.linux_setlease_fn = vfs_not_implemented_linux_setlease,
 	.getlock_fn = vfs_not_implemented_getlock,
@@ -1033,7 +1150,7 @@ static struct vfs_fn_pointers vfs_not_implemented_fns = {
 	.set_compression_fn = vfs_not_implemented_set_compression,
 
 	.fstreaminfo_fn = vfs_not_implemented_fstreaminfo,
-	.get_real_filename_fn = vfs_not_implemented_get_real_filename,
+	.get_real_filename_at_fn = vfs_not_implemented_get_real_filename_at,
 	.connectpath_fn = vfs_not_implemented_connectpath,
 	.brl_lock_windows_fn = vfs_not_implemented_brl_lock_windows,
 	.brl_unlock_windows_fn = vfs_not_implemented_brl_unlock_windows,
