@@ -172,6 +172,7 @@ static void smbd_smb2_request_getinfo_done(struct tevent_req *subreq)
 		/* Return a specific error with data. */
 		error = smbd_smb2_request_error_ex(req,
 						call_status,
+						0,
 						&out_output_buffer,
 						__location__);
 		if (!NT_STATUS_IS_OK(error)) {
@@ -297,12 +298,38 @@ static struct tevent_req *smbd_smb2_getinfo_send(TALLOC_CTX *mem_ctx,
 		struct timespec write_time_ts;
 		struct file_id fileid;
 		struct ea_list *ea_list = NULL;
-		int lock_data_count = 0;
-		char *lock_data = NULL;
 		size_t fixed_portion;
 
 		ZERO_STRUCT(write_time_ts);
 
+		/*
+		 * MS-SMB2 3.3.5.20.1 "Handling SMB2_0_INFO_FILE"
+		 *
+		 * FileBasicInformation, FileAllInformation,
+		 * FileNetworkOpenInformation, FileAttributeTagInformation
+		 * require FILE_READ_ATTRIBUTES.
+		 *
+		 * FileFullEaInformation requires FILE_READ_EA.
+		 */
+		switch (in_file_info_class) {
+		case FSCC_FILE_BASIC_INFORMATION:
+		case FSCC_FILE_ALL_INFORMATION:
+		case FSCC_FILE_NETWORK_OPEN_INFORMATION:
+		case FSCC_FILE_ATTRIBUTE_TAG_INFORMATION:
+			if (!(fsp->access_mask & SEC_FILE_READ_ATTRIBUTE)) {
+				tevent_req_nterror(req, NT_STATUS_ACCESS_DENIED);
+				return tevent_req_post(req, ev);
+			}
+			break;
+
+		case FSCC_FILE_FULL_EA_INFORMATION:
+			if (!(fsp->access_mask & SEC_FILE_READ_EA)) {
+				tevent_req_nterror(req, NT_STATUS_ACCESS_DENIED);
+				return tevent_req_post(req, ev);
+			}
+			break;
+		}
+
 		switch (in_file_info_class) {
 		case FSCC_FILE_FULL_EA_INFORMATION:
 			file_info_level = SMB2_FILE_FULL_EA_INFORMATION;
@@ -312,6 +339,14 @@ static struct tevent_req *smbd_smb2_getinfo_send(TALLOC_CTX *mem_ctx,
 			file_info_level = SMB2_FILE_ALL_INFORMATION;
 			break;
 
+		case SMB2_FILE_POSIX_INFORMATION:
+			if (!(fsp->posix_flags & FSP_POSIX_FLAGS_OPEN)) {
+				tevent_req_nterror(req, NT_STATUS_INVALID_LEVEL);
+				return tevent_req_post(req, ev);
+			}
+			file_info_level = SMB2_FILE_POSIX_INFORMATION_INTERNAL;
+			break;
+
 		default:
 			/* the levels directly map to the passthru levels */
 			file_info_level = in_file_info_class + 1000;
@@ -340,29 +375,17 @@ static struct tevent_req *smbd_smb2_getinfo_send(TALLOC_CTX *mem_ctx,
 			 * handle (returned from an NT SMB). NT5.0 seems
 			 * to do this call. JRA.
 			 */
-
-			if (fsp->fsp_name->flags & SMB_FILENAME_POSIX_PATH) {
-				/* Always do lstat for UNIX calls. */
-				if (SMB_VFS_LSTAT(conn, fsp->fsp_name)) {
-					DEBUG(3,("smbd_smb2_getinfo_send: "
-						 "SMB_VFS_LSTAT of %s failed "
-						 "(%s)\n", fsp_str_dbg(fsp),
-						 strerror(errno)));
-					status = map_nt_error_from_unix(errno);
-					tevent_req_nterror(req, status);
-					return tevent_req_post(req, ev);
-				}
-			} else if (SMB_VFS_STAT(conn, fsp->fsp_name)) {
-				DEBUG(3,("smbd_smb2_getinfo_send: "
-					 "SMB_VFS_STAT of %s failed (%s)\n",
+			int ret = vfs_stat(conn, fsp->fsp_name);
+			if (ret != 0) {
+				DBG_NOTICE("vfs_stat of %s failed (%s)\n",
 					 fsp_str_dbg(fsp),
-					 strerror(errno)));
+					 strerror(errno));
 				status = map_nt_error_from_unix(errno);
 				tevent_req_nterror(req, status);
 				return tevent_req_post(req, ev);
 			}
 
-			if (lp_smbd_getinfo_ask_sharemode(SNUM(conn))) {
+			if (fsp_getinfo_ask_sharemode(fsp)) {
 				fileid = vfs_file_id_from_sbuf(
 					conn, &fsp->fsp_name->st);
 				get_file_infos(fileid, fsp->name_hash,
@@ -374,15 +397,15 @@ static struct tevent_req *smbd_smb2_getinfo_send(TALLOC_CTX *mem_ctx,
 			 * Original code - this is an open file.
 			 */
 
-			if (SMB_VFS_FSTAT(fsp, &fsp->fsp_name->st) != 0) {
+			status = vfs_stat_fsp(fsp);
+			if (!NT_STATUS_IS_OK(status)) {
 				DEBUG(3, ("smbd_smb2_getinfo_send: "
 					  "fstat of %s failed (%s)\n",
-					  fsp_fnum_dbg(fsp), strerror(errno)));
-				status = map_nt_error_from_unix(errno);
+					  fsp_fnum_dbg(fsp), nt_errstr(status)));
 				tevent_req_nterror(req, status);
 				return tevent_req_post(req, ev);
 			}
-			if (lp_smbd_getinfo_ask_sharemode(SNUM(conn))) {
+			if (fsp_getinfo_ask_sharemode(fsp)) {
 				fileid = vfs_file_id_from_sbuf(
 					conn, &fsp->fsp_name->st);
 				get_file_infos(fileid, fsp->name_hash,
@@ -399,8 +422,6 @@ static struct tevent_req *smbd_smb2_getinfo_send(TALLOC_CTX *mem_ctx,
 					       delete_pending,
 					       write_time_ts,
 					       ea_list,
-					       lock_data_count,
-					       lock_data,
 					       STR_UNICODE,
 					       in_output_buffer_length,
 					       &fixed_portion,
